/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

// the default Matisse C++ header
#include "matisseCXX.h"

// the namespaces that contain the Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// A simple program that opens a read-only connection to a "snapshot" of the
// database at the time the connection is opened.  For more information, see
// the "Version Access" section of Getting Started with Matisse.
int main(int ac, char **av)
{
  if (ac < 3)
    {
      std::cerr << "Usage: " << av[0] << " <host> <db>" << std::endl;
      return -1;
    }
  try
    {
      MtDatabase db(av[1], av[2]);

      // open, select and start access to the database
      db.open();
      db.startVersionAccess();

      // version connect implies read-only access
      std::cout << "Successful connection and version access to "
		<< db << std::endl;

      db.endVersionAccess();
      db.close();
    }
  catch (MtException &e)
    {
      std::cerr << e << std::endl;
    }
  catch (...)
    {
      std::cerr << "Unknown exception" << std::endl;
    }
  return 0;
}
