/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

// the default Matisse C++ header
#include "matisseCXX.h"

// the namespaces that contain the Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// Shows how to create named versions when committing transactions and how to
// iterate through a list of versions.  For additional information, see 
// the "Named Versions" section of Getting Started with Matisse.
// define a local function in unnamed namespace
namespace {

// simple function which opens an iterator on the list of version names 
void listVersions(MtDatabase& db)
{
    MtStringIterator viter = db.versionIterator();

    std::cout << db << " has these versions:" << std::endl;
    while (viter.hasNext())
    {
        std::string vname = viter.next();
        std::cout << "\t" << vname << std::endl;
    }

	// close when finished.
    viter.close();
}

} // end anon namespace

int main(int ac, char **av)
{
  if (ac < 3)
    {
      std::cerr << "Usage: " << av[0] << " <host> <db>" << std::endl;
      return -1;
    }
  try
    {
      MtDatabase db(av[1], av[2]);

      // open, select and start access to the database
      db.open();

      db.startTransaction();
      std::cout << "\nVersions before regular commit:" << std::endl;
      listVersions(db);
      db.commit();

      db.startTransaction();
      std::cout << "\nVersions after regular commit:" << std::endl;
      listVersions(db);

      // Providing a prefix version name when committing a transaction
      // creates a named version of the database. The version name 
      // returned is the prefix name concatated to a unique ID  
      // appended by the server. A named version preserved a snapshot of 
      // the database and must be explicitly deleted.
      // (see mt_version command)
      std::string vername = db.commit("test");
      std::cout << "\nCommit and preserve version named: " << vername << std::endl;
        
      db.startVersionAccess();
      std::cout << "\nVersions after named commit:" << std::endl;
      listVersions(db);
      db.endVersionAccess();

      // A saved version can be accessed read-only
      db.startVersionAccess(vername);
      std::cout << "\nSuccessful access within version: " << vername
		<< std::endl;
      db.endVersionAccess();

      db.close();
    }
  catch (MtException &e)
    {
      std::cerr << e << std::endl;
    }
  catch (...)
    {
      std::cerr << "Unknown exception" << std::endl;
    }
  return 0;
}
