
##
## requires sun make
## requires MATISSE_HOME to be set to install directory
##

MV=/bin/mv
RM=/bin/rm -f
CP=/bin/cp

# lib for matisse
MATISSE_LIBS=-lmatisse
# lib for Matisse Lite
#MATISSE_LIBS=-lmatisselite
CPPFLAGS=-I${MATISSE_HOME}/include 
# with inline functions
#CPPFLAGS=-I${MATISSE_HOME}/include -DMT_USE_INLINE

LDFLAGS=-L${MATISSE_HOME}/lib

# sun CC compiler

CCLIB_OPT=-library=Cstd,Crun -staticlib=Cstd,Crun
CXX=CC

# debug version
#CXXFLAGS = -g -mt  -instances=static ${CCLIB_OPT} -DDEBUG
CXXFLAGS = -xO2 -mt  -instances=static ${CCLIB_OPT}


LINK=$(CXX) -mt ${CCLIB_OPT}
AR=$(CXX)

# to find matisseCXX.cpp
VPATH=$(MATISSE_HOME)/include

ODL_SRCS = company.cpp
ODL_FILE = company.odl

## the load application
load_SRCS=\
	$(ODL_SRCS)\
	matisseCXX.cpp\
	load.cpp

load_OBJS=$(load_SRCS:.cpp=.o)
load_EXEC=load


## the delete application
delete_SRCS=\
	$(ODL_SRCS)\
	matisseCXX.cpp\
	delete.cpp

delete_OBJS=$(delete_SRCS:.cpp=.o)
delete_EXEC=delete

## the newman1 application
newman1_SRCS=\
	$(ODL_SRCS)\
	matisseCXX.cpp\
	newman1.cpp

newman1_OBJS=$(newman1_SRCS:.cpp=.o)
newman1_EXEC=newman1


## the newman2 application
newman2_SRCS=\
	$(ODL_SRCS)\
	matisseCXX.cpp\
	newman2.cpp

newman2_OBJS=$(newman2_SRCS:.cpp=.o)
newman2_EXEC=newman2

ALL_OBJS =$(load_OBJS) $(delete_OBJS) $(newman1_OBJS) $(newman2_OBJS)
ALL_EXECS=$(load_EXEC) $(delete_EXEC) $(newman1_EXEC) $(newman2_EXEC)

default: $(ALL_EXECS)

# so the sources are always upto date..
$(ODL_SRCS): $(ODL_FILE)
	mt_sdl stubgen --lang cxx -f $(ODL_FILE)

$(load_EXEC) : $(load_OBJS)
	$(LINK) -o $@ $(load_OBJS) ${LDFLAGS} $(MATISSE_LIBS)

$(delete_EXEC) : $(delete_OBJS)
	$(LINK) -o $@ $(delete_OBJS) ${LDFLAGS} $(MATISSE_LIBS)

$(newman1_EXEC) : $(newman1_OBJS)
	$(LINK) -o $@ $(newman1_OBJS) ${LDFLAGS} $(MATISSE_LIBS)

$(newman2_EXEC) : $(newman2_OBJS)
	$(LINK) -o $@ $(newman2_OBJS) ${LDFLAGS} $(MATISSE_LIBS)

clean:
	$(RM) ${ALL_OBJS} $(ALL_EXECS)


%.o : %.cpp
	$(CXX) -c $(CPPFLAGS) $(CXXFLAGS) $< -o $@
