/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#include "matisseCXX.h"

#include "company.h"

#include <iostream>
#include <sstream>
#include <iomanip>

// namespaces containing Matisse classes
using namespace matisse;
using namespace std;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::chaps_10_11;

// Sample application demonstrating the most efficient way to allocate a
// large number of objects in one transaction:  create all the instances
// in a single server call, use this array of objects to set individual
// attributes, then commit. The drawback is that if commit fails (for
// example, due to a non-unique ID), no instances are created.
//
// Usage
//  load <hostname> <database> will load 100 Employees starting at id#0
//   additional arguments for number of objects to create and starting
//   ID are optional.
//
//  For each employee created a name is generated based on ID:  first
//  name is Annnnnn, lastname = Bnnnnnn where nnnnnn is id number padded
//  with 0's for width=6.  ID is guaranteed to be unique because the
//  unique_key option is set (see attribute declaration in company.odl).

int main(int ac, char **av)
{
  if (ac < 3)
  {
    std::cerr << "Usage: " << av[0]
              << " <host> <db> [<numcreate> <startID>]"
              << endl;
    return -1;
  }

  try
  {
    // In this example, there is a direct namespace mapping between 
    // the C++ namespace and the Schema namespace, so the default factory can handle it
    MtDatabase db(av[1], av[2]);

    int count = 100;
    int idstart = 0;

    if (ac > 3)
      count = ::atoi(av[3]);
    if (ac > 4)
      idstart = ::atoi(av[4]);

    db.open();
    db.startTransaction();

    std::cout << "Creating " << count << " Employees" << std::endl;

    // create an array of new instances
    MtObjectArray<Employee> emps(
      (Employee::getClass(db)).createInstances(count).typeless());
    for (int i = 0; i < count; i++)
    {
      Employee &e = emps[i];
            
      std::ostringstream os1;
      os1 << "JOHN";

      std::ostringstream os2;
      os2 << "DOE" << std::setw(6) << std::setfill('0') << i + idstart;

      e.setFirstName(os1.str());
      e.setLastName(os2.str());
      e.setId(i + idstart);

      std::cout << e.getMtClass().getMtFullName() << " " << e.getId() << " " << e.getFirstName() << " " << e.getLastName()  << std::endl;
    }
    std::cout << "Done" << std::endl;

    db.commit();
    db.close();

  }
  catch (MtException& mte)
  {
    std::cerr << "Matisse exception: " << mte << std::endl;
  }
  catch (std::exception& e)
  {
    std::cerr << "Standard exception: " << e.what() << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }

  return 0;
}

