/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

// the required Matisse C++ header
#include "matisseCXX.h"
#include "Person.h"

// namespaces containing Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::chaps_6_7_8;

// Sample application using an index for lookup and sorted listing.
int main(int ac, char **av)
{
    if (ac < 5)
    {
        std::cerr << "Usage: " << av[0]
                  << " <host> <db> <firstName> <lastName>"
                  << std::endl;
        return -1;
    }
    try
    {
      // In this example, there is a direct namespace mapping between 
      // the C++ namespace and the Schema namespace, so the default factory can handle it
        MtDatabase db(av[1], av[2]);

        db.open();

        // add some Person objects
        db.startTransaction();
        Person& p1 = Person::create(db);
        p1.setFirstName("John");
        p1.setLastName("Jones");
        Person& p2 = Person::create(db);
        p2.setFirstName("Fred");
        p2.setLastName("Jones");
        Person& p3 = Person::create(db);
        p3.setFirstName("John");
        p3.setLastName("Murray");
        Person& p4 = Person::create(db);
        p4.setFirstName("Fred");
        p4.setLastName("Flintstone");
    

        std::cout << "\nAdding 4 objects" << std::endl;
        db.commit();

        db.startVersionAccess();

        // just show the instances available
        std::cout << "\nPersons available:" << std::endl;
        MtObjectIterator<Person> pIter = Person::instanceIterator(db);
        while (pIter.hasNext())
        {
            Person &p = pIter.next();
            std::cout << "\t" << p.getFirstName() << " " << p.getLastName()
                      << std::endl;
        }


        // search for a Person with specified first last name
        std::string firstName = av[3];
        std::string lastName = av[4];

        std::cout << "\nLooking for : " << firstName << " " << lastName
                  << std::endl;

        // The lookup function must return a Person* to allow for NULL
        // to represent no match
        Person *found = Person::lookupPersonName(db, lastName, firstName);
        if (found != NULL)
        {
            std::cout << "\tfound " << found->getFirstName() << " "
                      << found->getLastName() << std::endl;
        }
        else
        {
            std::cout << "\tNobody found" << std::endl;
        }

        // instead of searching, open an iterator within the specified
        // criteria; an Index can specify upto 4 criteria and this API
        // would change accordingly (see examples.odl for specification)
        std::string fromFirstName = "Fred";
        std::string toFirstName = "John";
        std::string fromLastName = "Jones";
        std::string toLastName = "Murray";
        std::cout << "\nLookup Index from \"" << fromFirstName << " "
                  << fromLastName << "\" to \"" << toFirstName << " "
                  << toLastName << "\"" << std::endl;
        MtObjectIterator<Person> ppIter = Person::personNameIterator(db,
          fromLastName, fromFirstName, toLastName, toFirstName);
        while (ppIter.hasNext())
        {
            Person &p = ppIter.next();
            std::cout << "\t" << p.getFirstName() << " " << p.getLastName()
                      << std::endl;
        }
        MtValue vals[MT_MAX_CRITERIA] = {newString(fromLastName), newString(fromFirstName)};
	    MtSize num = Person::getPersonNameIndex(db).getObjectNumber(vals);
        std::cout << "\n" << num << " object(s) retrieved" << std::endl;
        
	    MtSize count = Person::getPersonNameIndex(db).getIndexEntriesNumber();
        std::cout << "\n" << count << " entries in the index" << std::endl;

        db.endVersionAccess();

        db.startTransaction();

        std::cout << "\nRemoving objects..." << std::endl;
        p1.remove();
        p2.remove();
        p3.remove();
        p4.remove();
        db.commit();
          
        db.close();
    }
    catch (MtException &e)
    {
        std::cerr << e << std::endl;
    }
    catch (...)
    {
        std::cerr << "Unknown exception" << std::endl;
    }
    return 0;
}       

